# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
## ROLLING-ORIGIN CROSS VALIDATION WITH ADJUSTED YEAR INDEX ====================
# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

out_of_sample <- function(data = NULL,  # dataset
                          response = NULL,  # response variable
                          vars = NULL,  # model covariates
                          res = NULL,  # random effects
                          lags = NULL,  # lead time
                          labels = NULL,
                          start_id = 1,  # index to start cross validation
                          save = FALSE,  # save output
                          source_path = getwd(),
                          file_path = getwd(),
                          file_name = "pred",  # file name
                          tmp = TRUE,  # save temporary output
                          nsamples = 1000,  # number of posterior samples
                          epi_thresh = "quantile", # options: 'quantile', 'mean'
                          q = 0.75, # quantile value (0, 1) for 'quantile'
                          sd = 1, # standard deviation value for 'mean', int >= 0
                          temp = NULL){
  
  # read in dependent functions
  source(paste0(source_path, "/run_inla.R"))
  source(paste0(source_path, "/outbreak_probability.R"))
  source(paste0(source_path, "/predictive_ability.R"))
  
  # create directories
  if (file.exists(paste0(file_path, "/tmp"))){
    invisible()
  } else {
    dir.create(paste0(file_path, "/tmp"))
  }
  
  if (file.exists(paste0(file_path, "/cross_validation"))){
    invisible()
  } else {
    dir.create(paste0(file_path, "/cross_validation"))
  }
  
  # create object to store outputs
  if (tmp){
    temp <- list(posterior_predictive = list(),
                 outbreak_probability = list(),
                 predictive_ability = list())
  }
  
  # error message
  stopifnot("epi_thresh parameter invalid. Options are 'quantile', 'mean'" =
              epi_thresh == "quantile" | epi_thresh == "mean")
  
  stopifnot("q parameter invalid. Please choose quantile value greater than 0 and less than 1." =
              q < 1 | q > 0)
  
  stopifnot("sd parameter invalid. Please choose positive integer value for standard deviation." =
              is.numeric(sd) & sd == as.integer(sd) & sd >= 0)
  
  # stopifnot("res parameter invalid. List should either be length 1 or length of vars." =
  #             length(res) == 1 | length(res) == length(vars))
  
  ntimes <- nrow(data)  # number of months
  npars <- 2  # distribution mean and overdispersion parameter (neg binomial)

  # loop over each model  
  for (j in 1:length(vars)) {
    
    v <- vars[[j]]
    l <- lags[[j]]
    
    if(length(res) > 1){res_i <- res[[j]]} else{res_i <- res}
    
    npreds <- ntimes - start_id + 1
    posterior_predictive <- matrix(NA, npreds, nsamples)
    
    # loop over each month
    for (i in start_id:ntimes){
      
      # remove subsequent information from data
      data.pred <- data[1:i, ]
      
      # re-adjust the dataset around the prediction model
      y.val <- ifelse((i + 1) %% 12 == 0, 12, (i + 1) %% 12)
      nyear <- nrow(data.pred[data.pred$cal_month == data.pred[i, "cal_month"], ])
      
      if (y.val != 1){
        data.pred$year_index <- c(rep(1, y.val-1),
                                rep(2:(nyear + 1), each = 12,
                                    length.out = (nrow(data.pred) - y.val + 1)))
      }
      
      # remove response from training dataset
      data.pred <- data.pred %>% 
        mutate(cases = ifelse(time >= i-l, NA, cases))
      
      # run inla model
      form <- reformulate(c(1, na.omit(unlist(res_i)), na.omit(v)), response)
      m <- run_mod(form, data.pred)
      
      # samples from the posterior marginal distributions of the parameters
      posterior_samples <- inla.posterior.sample(nsamples, m)
      
      # extract samples of parameters of interest
      par_samples <- matrix(NA, npars, nsamples)
      
      for (col in 1:nsamples){
        par_samples[1, col] <- posterior_samples[[col]][["hyperpar"]][[1]]  # overdispersion
        par_samples[2, col] <- posterior_samples[[col]][["latent"]][[i]]  # target month
      }
      
      # posterior predictive distribution
      predictive_out <- array(NA, nsamples)
      
      for (sample in 1:nsamples){
        par_sample <- par_samples[, sample]
        predictive_out[sample] <- rnbinom(1,
                                          mu = exp(par_sample[-1]), 
                                          size = par_sample[1])
      }
      
      # save posterior predictive
      k <- i - start_id + 1
      posterior_predictive[k, ] <- predictive_out
      
      # month complete
      cat('\r', paste0("Model ", j, " of ", length(vars), " - Target month ", i,
                       " of ", ntimes))
    }
    
    # model complete
    msg <- paste0("Completed model ", j, " of ", length(vars),
                  paste(rep(" ", 15), collapse = ""))
    cat('\r', msg)
    
    # set label
    if (is.null(labels)){
      lab <- ifelse(is.na(v), "no_covars", v)
    } else{
      lab <- labels[j]
    }
    
    # save samples from the posterior predictive distribution
    temp[["posterior_predictive"]][[lab]] <- posterior_predictive
    
    # compute outbreak probability
    outbreak_data <- outbreak_probability(data, 
                                          samples = posterior_predictive,
                                          epi_floor = 5,
                                          epi_thresh = epi_thresh,
                                          q = q,
                                          sd = sd,
                                          response = response,
                                          start_id = start_id)
    
    temp[["outbreak_probability"]][[lab]] <- outbreak_data
    
    # Indicators of predictive ability
    temp[["predictive_ability"]] <- bind_rows(temp[["predictive_ability"]],
                                              predictive_ability(outbreak_data,
                                                                 posterior_predictive,
                                                                 label = lab,
                                                                 response = response))
    
    # Save preliminary output in tmp folder
    saveRDS(temp, paste0(file_path, "/tmp/temp.rds"))
    
  }
  
  if(save) saveRDS(temp, paste0(file_path, "/cross_validation/", file_name, ".rds"))
  if(j == length(vars)) unlink(paste0(file_path, "/tmp"), recursive=TRUE)
  
  return(temp)
  
}


## END