# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
## EPIDEMIC THRESHOLD SENSITIVITY ANALYSIS (CROSS VALIDATION) ==================

# Description:
#     Sensitivity analysis: Cross validation for different epidemic thresholds,
#     including 50th percentile (median), 75th percentile and 95th percentile,
#     mean, mean + 1 standard deviation, mean + 2 standard deviations.

# Paper:
#     Compound and cascading effects of climatic extremes on dengue outbreak
#     risk in the Caribbean: an impact-based modelling framework with long-lag
#     and short-lag interactions

# Script authors:
#     Chloe Fletcher        ORCID: 0000-0002-6705-7605
#     Prof. Rachel Lowe     ORCID: 0000-0003-3939-7343

# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

## Source packages, functions and data -----------------------------------------

# load packages and functions
source("functions/00_packages_functions.R")
source("functions/outbreak_probability.R")
source("functions/predictive_ability.R")

# read in harmonised data
data <- read.csv("data/barbados_data.csv")

# set start year, start month and start index
startyear <- data$cal_year[1]
startmonth <- data$cal_month[1]
startid <- nrow(data) - 12 * 10 + 1  # last 10 years of dataset

# load model output
spi6.mod.out <- readRDS("outputs/spi6_mods.rds")
candidate_mods <- c(23, 29, 30, 58, 59, 138, 173)

# model covariates
vars.b <- c(NA)
vars.p <- spi6.mod.out$mod.gof$vars[candidate_mods]


## Change epidemic threshold for existing CV models ----------------------------

# read in best models
pred.b <- readRDS("outputs/cross_validation/preds_b.rds")
pred.i.ay <- readRDS("outputs/cross_validation/preds_i_adjustedyear_logdir.rds")

# update baseline models with different epi thresholds
# (change the threshold manually so posterior samples remain consistent)
# pred.b.q50 <- update.epi.thresh.cv(vars.b, pred.b, epi_thresh = "quantile",
#                                    q = 0.5, file_path = "preds_b_50pct")
# pred.b.q95 <- update.epi.thresh.cv(vars.b, pred.b, epi_thresh = "quantile",
#                                    q = 0.95, file_path = "preds_b_95pct")
# pred.b.m0s <- update.epi.thresh.cv(vars.b, pred.b, epi_thresh = "mean",
#                                    sd = 0, file_path = "preds_b_mean")
# pred.b.m1s <- update.epi.thresh.cv(vars.b, pred.b, epi_thresh = "mean",
#                                    sd = 1, file_path = "preds_b_mean+1sd")
# pred.b.m2s <- update.epi.thresh.cv(vars.b, pred.b, epi_thresh = "mean",
#                                    sd = 2, file_path = "preds_b_mean+2sd")

# update interaction models with different epi thresholds
# (change the threshold manually so posterior samples remain consistent)
# pred.i.q50 <- update.epi.thresh.cv(vars.p, pred.i.ay, epi_thresh = "quantile",
#                                    q = 0.5, file_path = "preds_i_50pct")
# pred.i.q95 <- update.epi.thresh.cv(vars.p, pred.i.ay, epi_thresh = "quantile",
#                                    q = 0.95, file_path = "preds_i_95pct")
# pred.i.m0s <- update.epi.thresh.cv(vars.p, pred.i.ay, epi_thresh = "mean",
#                                    sd = 0, file_path = "preds_i_mean")
# pred.i.m1s <- update.epi.thresh.cv(vars.p, pred.i.ay, epi_thresh = "mean",
#                                    sd = 1, file_path = "preds_i_mean+1sd")
# pred.i.m2s <- update.epi.thresh.cv(vars.p, pred.i.ay, epi_thresh = "mean",
#                                    sd = 2, file_path = "preds_i_mean+2sd")


## Tabulate results ------------------------------------------------------------

# read in baseline models
pred.b.q50 <- readRDS("supplementary/outputs/cross_validation/preds_b_50pct.rds")
pred.b.q95 <- readRDS("supplementary/outputs/cross_validation/preds_b_95pct.rds")
pred.b.m0s <- readRDS("supplementary/outputs/cross_validation/preds_b_mean.rds")
pred.b.m1s <- readRDS("supplementary/outputs/cross_validation/preds_b_mean+1sd.rds")
pred.b.m2s <- readRDS("supplementary/outputs/cross_validation/preds_b_mean+2sd.rds")

# read in interaction models
pred.i.q50 <- readRDS("supplementary/outputs/cross_validation/preds_i_50pct.rds")
pred.i.q95 <- readRDS("supplementary/outputs/cross_validation/preds_i_95pct.rds")
pred.i.m0s <- readRDS("supplementary/outputs/cross_validation/preds_i_mean.rds")
pred.i.m1s <- readRDS("supplementary/outputs/cross_validation/preds_i_mean+1sd.rds")
pred.i.m2s <- readRDS("supplementary/outputs/cross_validation/preds_i_mean+2sd.rds")

# tabulate hit rate, false alarm rate, auc, trigger and crps
threshs <- c("50th pct", "75th pct", "95th pct", "mean", "mean + 1 sd", "mean + 2 sd")

tab <- rbind(pred.b.q50$predictive_ability, pred.b$predictive_ability, 
             pred.b.q95$predictive_ability, pred.b.m0s$predictive_ability, 
             pred.b.m1s$predictive_ability, pred.b.m2s$predictive_ability, 
             pred.i.q50$predictive_ability, pred.i.ay$predictive_ability, 
             pred.i.q95$predictive_ability, pred.i.m0s$predictive_ability, 
             pred.i.m1s$predictive_ability, pred.i.m2s$predictive_ability)

tab$model <- c(paste("current practice:", threshs),
               paste0(rep(vars.p, length(threshs)), ": ",
                      rep(threshs, each = length(vars.p))))

tab <- tab[with(tab, order(model)), ]
row.names(tab) <- NULL

# calculate number of outbreaks depending on threshold
tab$outbreaks <- c(sum(pred.b.q50$outbreak_probability[[1]]$obs_outbreak),
                   sum(pred.b$outbreak_probability[[1]]$obs_outbreak), 
                   sum(pred.b.q95$outbreak_probability[[1]]$obs_outbreak),
                   sum(pred.b.m0s$outbreak_probability[[1]]$obs_outbreak), 
                   sum(pred.b.m1s$outbreak_probability[[1]]$obs_outbreak),
                   sum(pred.b.m2s$outbreak_probability[[1]]$obs_outbreak), 
                   rep(c(sum(pred.i.q50$outbreak_probability[[1]]$obs_outbreak),
                         sum(pred.i.ay$outbreak_probability[[1]]$obs_outbreak), 
                         sum(pred.i.q95$outbreak_probability[[1]]$obs_outbreak),
                         sum(pred.i.m0s$outbreak_probability[[1]]$obs_outbreak), 
                         sum(pred.i.m1s$outbreak_probability[[1]]$obs_outbreak),
                         sum(pred.i.m2s$outbreak_probability[[1]]$obs_outbreak)),
                       length(vars.p)))

# supplementary table 4
tab_st04 <- tab[7:12, ]


## Plot heatmap for different epi thresholds -----------------------------------

# final model selected
i <- 4

# plot outbreak probability
fs <- 18

g1 <- plot.prob.outbreak.cv(pred.i.q50, data[startid, "cal_year"], startmonth, i,
                            threshold = pred.i.q50$predictive_ability[i, "trigger"]) +
  ggtitle("50th Percentile (Median)") +
  theme(plot.title = element_text(size = fs))

g2 <- plot.prob.outbreak.cv(pred.i.m0s, data[startid, "cal_year"], startmonth, i,
                            threshold = pred.i.m0s$predictive_ability[i, "trigger"]) +
  ggtitle("Mean") +
  theme(plot.title = element_text(size = fs))

g3 <- plot.prob.outbreak.cv(pred.i.ay, data[startid, "cal_year"], startmonth, i,
                            threshold = pred.i.ay$predictive_ability[i, "trigger"]) +
  ggtitle("75th Percentile") +
  theme(plot.title = element_text(size = fs))

g4 <- plot.prob.outbreak.cv(pred.i.m1s, data[startid, "cal_year"], startmonth, i,
                            threshold = pred.i.m1s$predictive_ability[i, "trigger"]) +
  ggtitle("Mean + 1 Standard Deviation") +
  theme(plot.title = element_text(size = fs))

g5 <- plot.prob.outbreak.cv(pred.i.q95, data[startid, "cal_year"], startmonth, i,
                            threshold = pred.i.q95$predictive_ability[i, "trigger"]) +
  ggtitle("95th Percentile") +
  theme(plot.title = element_text(size = fs))

g6 <- plot.prob.outbreak.cv(pred.i.m2s, data[startid, "cal_year"], startmonth, i,
                            threshold = pred.i.m2s$predictive_ability[i, "trigger"]) +
  ggtitle("Mean + 2 Standard Deviation") +
  theme(plot.title = element_text(size = fs))

# assemble as single plot
ggdraw() +
  draw_plot(g1, x = 0, y = 2/3, width = .5, height = 1/3) +
  draw_plot(g2, x = .5, y = 2/3, width = .5, height = 1/3) +
  draw_plot(g3, x = 0, y = 1/3, width = .5, height = 1/3) +
  draw_plot(g4, x = .5, y = 1/3, width = .5, height = 1/3) +
  draw_plot(g5, x = 0, y = 0, width = .5, height = 1/3) +
  draw_plot(g6, x = .5, y = 0, width = .5, height = 1/3) +
  draw_plot_label(label = c("A", "B", "C", "D", "E", "F"), size = 18,
                  x = c(0, .5, 0, .5, 0, .5),
                  y = c(1, 1, 2/3, 2/3, 1/3, 1/3)) +
  theme_void() +
  theme(plot.background = element_rect(fill = "white")) +
  panel_border(remove = TRUE)
#ggsave("outputs/figs/SF_09.png", width = 13, height = 15, dpi = 300)


## END